#include "Holes_dlg.h"
#include "form3.h"

#include <math.h>
#include <qmessagebox.h>

using namespace ratios;
// declared in form3.h contains pecking decrement ratios

void Form2::On_Quit_clicked()
{
    close();
}

void Form2::On_generate_clicked()
{
QString cstr;
double   X_start, Y_start, inter_pos_feed, peck_feed, peck_dwell;
double   Z_boring_feed, Z_inter_pos_height, Z_bore_depth, Z_start_height;
double   hole_spacing_X, hole_spacing_Y, row_offset_X, row_offset_Y, drill_diameter,  spindle_speed;
int     num_holes_row, num_rows;
int     linenum = 1;
int     block_start, block_end, f;
double  peckarray[20];
double  feedarray[20];
double  speedarray[20];
bool    b_coolant;
double  minRPM, minFeed, minPeck, maxPeck, curDepth, curFeed, curPeck, curSpeed, c, oldSpeed;

    // get data from line edits

    X_start = ((QString)le_X_start->text()).toDouble();
    Y_start = ((QString)le_Y_start->text()).toDouble();
    inter_pos_feed = ((QString)le_inter_position_feed->text()).toDouble();
    Z_boring_feed =  ((QString)le_Z_boring_feed->text()).toDouble();  
    Z_inter_pos_height = ((QString)le_Z_inter_position_height->text()).toDouble();
    Z_bore_depth = ((QString)le_Z_bore_depth->text()).toDouble();
    if(Z_bore_depth < 0)
        Z_bore_depth = Z_bore_depth * -1;   // make sure not neg number
    Z_start_height = ((QString)le_Z_start_height->text()).toDouble();
    peck_feed = ((QString)le_peck_feed->text()).toDouble();
    peck_dwell = ((QString)le_peck_dwell->text()).toDouble();
    hole_spacing_X = ((QString)le_hole_spacing_X->text()).toDouble();
    hole_spacing_Y = ((QString)le_hole_spacing_Y->text()).toDouble();
    row_offset_X = ((QString)le_row_offset_X->text()).toDouble();
    row_offset_Y = ((QString)le_row_offset_Y->text()).toDouble();
    drill_diameter = ((QString)le_drill_diameter->text()).toDouble();
    spindle_speed = ((QString)le_spindle_speed->text()).toDouble();

    num_rows = ((QString)le_num_rows->text()).toInt();
    num_holes_row = ((QString)le_num_holes_row->text()).toInt();
    
    oldSpeed = spindle_speed;
    b_coolant = cb_coolant->isChecked();

    if(X_start < 0 || Y_start < 0)
        {
        QMessageBox::information(this, "EASIMILL - Hole Pattern generator",
            "X and Y start co-ordinates can be Zero or a positive offset\n"
            "If you want to start at a negative offset, change your G55 offsets accordingly");
        return;
        }

    block_start = block_end = 0; 
    f = 0;
    // calculate from stored values
    minRPM = ceil(spindle_speed * minSpeedRatio);
    minFeed = ceil(Z_boring_feed * minFeedRatio);
    maxPeck = floor(drill_diameter * maxPeckRatio);
    if(maxPeck > Z_bore_depth)
          maxPeck = Z_bore_depth ;
    minPeck = ceil(drill_diameter * minPeckRatio);

// If peck drilling, fill arrays with calculated values to complete bore
    if(peck_feed > 0)
        {
        for(f = 0; f < 20; f++)
            {        
            peckarray[f] = 0;
            feedarray[f]= 0;
            speedarray[f] = 0;
            }

        f = c =0;
        curPeck =  curDepth = 0;
        do{
            curPeck = maxPeck;
            curDepth += maxPeck;
            peckarray[f] = curDepth;

            feedarray[f] = Z_boring_feed;
            curFeed = Z_boring_feed;
            speedarray[f] = spindle_speed;
            curSpeed = spindle_speed;

            f++;
            }while((curDepth + maxPeck) < (drill_diameter * 3));
        // fill arrays
        for(; curDepth < Z_bore_depth; f++)
            {
            c = curPeck * peckRatio;
            if( c < minPeck) c = minPeck;
            if((c + curDepth) > Z_bore_depth)
                {
                peckarray[f] = Z_bore_depth;
                c += 1; // ensure is over next loop
                }
            else
                peckarray[f] = c + curDepth;
            curDepth += c;
            curPeck = c;

            c =  ceil(curFeed * feedRatio);
            if( c < minFeed) c = minFeed;      
            feedarray[f] = c ;
            curFeed = c;
            c = ceil(curSpeed * speedRatio);
            if( c < minRPM) c = minRPM; 
            speedarray[f] = c;
            curSpeed = c;
            }
        }
///////   start strings

    cstr.sprintf("N%d M3 S%.0f \n",linenum++, spindle_speed);
    te_gcode->setText(cstr);
    cstr.sprintf("N%d G01 X%.3f Y%.3f Z%.3f FZ%.0f\n",linenum++, X_start, Y_start, Z_start_height, inter_pos_feed);
    te_gcode->append(cstr);

    if(b_coolant)
        {
        cstr.sprintf("N%d M8\n", linenum++);
        te_gcode->append(cstr);
        }    

////// drilling block

    block_start = linenum;

    // First line of block must contain co-ordinates for all axes as machine returns to this position
    // at every iteration +/- offsets in G81 block
    cstr.sprintf("N%d G01 X%.3f Y%.3f Z%.3f FZ%.0f\n",linenum++ ,X_start, Y_start, Z_inter_pos_height, inter_pos_feed);
    te_gcode->append(cstr);

    if(peck_feed > 0)
        {
        for(int b = 0; b < f ; b++)
            {
            // logic used in PCD drilling does not work in a G81 loop
            // Since loop is only written once, the spindle speed will be correct and not be repeated
            // however when repeating the loop in real time, spindle speed will remain stuck on lowest setting in loop
            if((speedarray[b] > minRPM && speedarray[b] != oldSpeed) || (b == 0))  
                {
                cstr.sprintf("N%d M3 S%.0f\n",linenum++ ,speedarray[b] );
                te_gcode->append(cstr);
                oldSpeed = speedarray[b] ;
                }
            cstr.sprintf("N%d G01 Z-%.3f FZ%.0f\n",linenum++ , peckarray[b], feedarray[b] );
            te_gcode->append(cstr);
            cstr.sprintf("N%d G01 Z%.3f FZ%.0f\n",linenum++ , Z_inter_pos_height, peck_feed );
            te_gcode->append(cstr);
            if(peckarray[b] >= Z_bore_depth)
                break;   // save repeat descent to full depth on last line of drill code
            cstr.sprintf("N%d G4 D%.0f\n",linenum++ , peck_dwell );
            te_gcode->append(cstr);
            cstr.sprintf("N%d G01 Z-%.3f FZ%.0f\n",linenum++ , peckarray[b], peck_feed );
            te_gcode->append(cstr);
            }
        }
    else
        {
        cstr.sprintf("N%d G01 Z-%.3f FZ%.0f\n",linenum++ , Z_bore_depth, Z_boring_feed );
        te_gcode->append(cstr);
        }
    
    block_end = linenum;

    cstr.sprintf("N%d G01 Z%.3f FZ%.0f\n",linenum++ , Z_inter_pos_height , inter_pos_feed ); // should never be necessary, but ensures cutter raised
    te_gcode->append(cstr);

    cstr.sprintf("N%d G81 R%d E%d N%d X%.3f Y%.3f FX%.0f\n",linenum++, block_start, block_end, num_holes_row - 1, 
                    hole_spacing_X, hole_spacing_Y, inter_pos_feed );
    te_gcode->append(cstr);

    if(num_rows > 1)
        {
        block_end = linenum - 1;
        cstr.sprintf("N%d G81 R%d E%d N%d X%.3f Y%.3f FX%.0f\n",linenum++, block_start, block_end, num_rows-1,
                    row_offset_X, row_offset_Y, inter_pos_feed );
        te_gcode->append(cstr);

        }


////////////    end strings

    cstr.sprintf("N%d G00 Z%.3f\n" , linenum++, Z_start_height);
    te_gcode->append(cstr);
    if(b_coolant)
        {
        cstr.sprintf("N%d M9\n", linenum++);
        te_gcode->append(cstr);
        }    
    cstr.sprintf("N%d M5\nM2\n", linenum++ ); 
    te_gcode->append(cstr);

    te_gcode->selectAll(true);
    te_gcode->copy();

    cstr.sprintf("\n\nThis code is now in the clipboard\n\n");
    te_gcode->append(cstr);   
}

