#include "PCD_dlg.h"
#include "form3.h"

#include <math.h>
#include <iostream> 
#include <qmessagebox.h>

#define PI 3.14159265

using namespace ratios;


void Form1::On_Quit_clicked()
{
    close();
}

void Form1::On_generate_clicked()
{
QString cstr;
double   X_centre, Y_centre, diameter, inter_pos_feed, peck_feed, peck_dwell;
double   Z_boring_feed, Z_inter_pos_height, Z_bore_depth, Z_start_height, drill_diameter, spindle_speed;
int     num_holes ;

double  calc_angle, start_angle, radius, x, y, x1, y1, quot, adj_angle;
bool    b_coolant;
int     linenum = 1;
int     f;
double  peckarray[50];
double  feedarray[50];
double  speedarray[50];

double  minRPM, minFeed, minPeck, maxPeck, curDepth, curFeed, curPeck, curSpeed, c, oldSpeed;

    // get data from line edits
    X_centre = ((QString)le_X_centre->text()).toDouble();
    Y_centre = ((QString)le_Y_centre->text()).toDouble();
    diameter = ((QString)le_pcd->text()).toDouble();
    start_angle = ((QString)le_start_angle->text()).toDouble();
    inter_pos_feed = ((QString)le_inter_position_feed->text()).toDouble();
    Z_boring_feed =  ((QString)le_Z_boring_feed->text()).toDouble();  
    Z_inter_pos_height = ((QString)le_Z_inter_position_height->text()).toDouble();
    Z_bore_depth = ((QString)le_Z_bore_depth->text()).toDouble();
    if(Z_bore_depth < 0)
        Z_bore_depth = Z_bore_depth * -1;   // make sure not neg number
    Z_start_height = ((QString)le_Z_start_height->text()).toDouble();
    peck_feed = ((QString)le_peck_feed->text()).toDouble();
    peck_dwell = ((QString)le_peck_dwell->text()).toDouble();
    drill_diameter = ((QString)le_drill_diameter->text()).toDouble();
    spindle_speed = ((QString)le_spindle_speed->text()).toDouble();

    num_holes = ((QString)le_num_holes->text()).toInt();
    oldSpeed = spindle_speed;
    b_coolant = cb_coolant->isChecked();

    if(X_centre < 0 || Y_centre < 0)
        {
        QMessageBox::information(this, "EASIMILL - PCD generator",
            "X and Y centre co-ordinates can be Zero or a positive offset\n"
            "If you want to start at a negative offset, change your G55 offsets accordingly");
        return;
        }

// If peck drilling, fill arrays with calculated values to complete bore
// calculate from stored values
    minRPM = ceil(spindle_speed * minSpeedRatio);
    minFeed = ceil(Z_boring_feed * minFeedRatio);
    maxPeck = floor(drill_diameter * maxPeckRatio);
    if(maxPeck > Z_bore_depth)
          maxPeck = Z_bore_depth ;
    minPeck = ceil(drill_diameter * minPeckRatio);
    
    // initialise
    calc_angle = (360 / num_holes);
    radius = (diameter / 2);
    f = 0;
// If peck drilling, fill arrays with calculated values to complete bore
    if(peck_feed > 0)
        {
        for(f = 0; f < 50; f++)
            {        
            peckarray[f] = 0;
            feedarray[f]= 0;
            speedarray[f] = 0;
            }

        f = c = 0;
        curPeck =  curDepth = 0;
        do{
            curPeck = maxPeck;
            curDepth += maxPeck;
            peckarray[f] = curDepth;

            feedarray[f] = Z_boring_feed;
            curFeed = Z_boring_feed;
            speedarray[f] = spindle_speed;
            curSpeed = spindle_speed;

            f++;
            }while((curDepth + maxPeck) < (drill_diameter * 3));
        // fill arrays
        for(; curDepth < Z_bore_depth; f++)
            {
            c = curPeck * peckRatio;
            if( c < minPeck) c = minPeck;
            if((c + curDepth) > Z_bore_depth)
                {
                peckarray[f] = Z_bore_depth;
                c += 1; // ensure is over next loop
                }
            else
                peckarray[f] = c + curDepth;
            curDepth += c;
            curPeck = c;

            c =  ceil(curFeed * feedRatio);
            if( c < minFeed) c = minFeed;      
            feedarray[f] = c ;
            curFeed = c;

            c = ceil(curSpeed * speedRatio);
            if( c < minRPM) c = minRPM; 
            speedarray[f] = c;
            curSpeed = c;
            }
        }

    ///////   start strings

    cstr.sprintf("N%d M3 S%.0f \n",linenum++, spindle_speed);
    te_gcode->setText(cstr);
    cstr.sprintf("N%d G01 X%.3f Y%.3f Z%.3f FZ%.0f\n",linenum++, X_centre, Y_centre, Z_start_height, inter_pos_feed);
    te_gcode->append(cstr);

    if(b_coolant)
        {
        cstr.sprintf("N%d M8\n", linenum++);
        te_gcode->append(cstr);
        }    

    for(int a = 0; a < num_holes; a++)
        {
        // C cos and sin functions work in radians so have to convert to get degrees
        // calculate the cumulative angle first and add start angle offset, if any
        // then multiply by radians formula to get quotient which is feed to cos() and sin()
        adj_angle = (calc_angle * a) + start_angle;
        if(adj_angle > 360)
            adj_angle = adj_angle - 360;  // cater for start_angle greater than calc_angle
        quot = (adj_angle * PI/180);
        x1 = radius * cos(quot);
        y1 = radius * sin(quot);

        x=(x1+X_centre);
        y=(y1+Y_centre);   
        cstr.sprintf("N%d G01 X%.3f Y%.3f Z%.3f FZ%.0f\n",linenum++, x, y, Z_inter_pos_height, inter_pos_feed);
        te_gcode->append(cstr);

        if(peck_feed > 0) // if peck drilling
            {
            for(int b = 0; b < f ; b++)
                {
                if(speedarray[b] > minRPM && speedarray[b] != oldSpeed)  // cut unnecessary S commands but ensure speed reset to max each iteration
                    {
                    cstr.sprintf("N%d M3 S%.0f\n",linenum++ ,speedarray[b] );
                    te_gcode->append(cstr);
                    oldSpeed = speedarray[b] ;
                    }
                cstr.sprintf("N%d G01 Z-%.3f FZ%.0f\n",linenum++ , peckarray[b], feedarray[b] );
                te_gcode->append(cstr);
                cstr.sprintf("N%d G01 Z%.3f FZ%.0f\n",linenum++ , Z_inter_pos_height, peck_feed );
                te_gcode->append(cstr);
                if(peckarray[b] < Z_bore_depth) // save repeat descent to full depth on last line of drill code
                    {    
                    cstr.sprintf("N%d G4 D%.0f\n",linenum++ , peck_dwell );
                    te_gcode->append(cstr);
                    cstr.sprintf("N%d G01 Z-%.3f FZ%.0f\n",linenum++ , peckarray[b], peck_feed );
                    te_gcode->append(cstr);
                    }
                }
            }
        else
            {
            cstr.sprintf("N%d G01 Z-%.3f FZ%.0f\n",linenum++ , Z_bore_depth, Z_boring_feed );
            te_gcode->append(cstr);
            cstr.sprintf("N%d G01 Z%.3f FZ%.0f\n",linenum++ , Z_inter_pos_height, inter_pos_feed );
            te_gcode->append(cstr);
            }
        } // end of for() loop
        
    cstr.sprintf("N%d G00 Z%.3f\n" , linenum++, Z_start_height);
    te_gcode->append(cstr);
    if(b_coolant)
        {
        cstr.sprintf("N%d M9\n", linenum++);
        te_gcode->append(cstr);
        }    
    cstr.sprintf("N%d M5\nM2\n", linenum++ ); 
    te_gcode->append(cstr);

    te_gcode->selectAll(true);
    te_gcode->copy();

    cstr.sprintf("\n\nThis code is now in the clipboard\n\n");
    te_gcode->append(cstr);
}


