#include "PNC3.h"
#include "PCD_dlg.h"
#include "Holes_dlg.h"
#include "Ring_dlg.h"
#include "form3.h"
#include "Rectangle_dlg.h"

#include <stdio.h>    /* Standard input/output definitions */
#include <stdlib.h> 
#include <stdint.h>   /* Standard types */
#include <string.h>   /* String function definitions */
#include <unistd.h>   /* UNIX standard function definitions */
#include <fcntl.h>    /* File control definitions */
#include <errno.h>    /* Error number definitions */
#include <termios.h>  /* POSIX terminal control definitions */
#include <sys/ioctl.h>
#include <iostream>


using namespace ratios;



void PNC3::initialise()
{
    defaultDir = "/root/Easimill/CNC_Files";
 	fd = 0;

	ack = "\x06";
	nak = "\x15";
   	uk = "\x31";

   // defaults for settings dialog (Form3)

    peckRatio =  0.75;
    feedRatio =  0.9;
    speedRatio =  0.95;
    minPeckRatio = 0.5;
    maxPeckRatio = 1;
    minSpeedRatio = 0.75;
    minFeedRatio =  0.5;

}

/*************************************************************************************/

void PNC3::fileNew()
{
    if(textEdit1->isModified())
		{
		fileSave();
		}
    textEdit1->setText("");
    setCaption( tr( "PNC3Comm - " ) );
    lastFilename = "";
}


////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::fileOpen()
{
QString buff, s1;
int linenum = 1;
QString filename = QFileDialog::getOpenFileName(
                    defaultDir,
                    "CNC Files (*.cnc *.CNC *.nc *.NC *.ngc *.NGC);;"
		    "All Files (*.*)",
                    this,
                    "Open File",
                    "Choose a file" );
	if(!filename.isEmpty())
		{
		QFile file( filename ); // Read the text from a file
		file.open(IO_ReadOnly);
		QTextStream stream(&file);

		blockList.clear();

		while(!stream.atEnd())
			{
			buff = stream.readLine();
			Format(buff, true);
			if(buff.length())
				{   // insert line numbers unles at M2
                if((buff.find("N", 0) == -1) && ( (buff.find("M2", 0) == -1) && (buff.find("M02", 0) == -1) ) )
                    {
                    s1 = QString("N%1 ")
                        .arg(linenum++, 0, 'f', 0);
                    buff = s1 + buff;
                    }
				blockList += buff;
				textEdit1->insert(buff);
				}
			}


		setCaption( tr( "PNC3Comm - " + filename ) );
        lastFilename = filename;
		}   	
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::fileSave()
{

	if(lastFilename.isEmpty())
		{
		fileSaveAs();
		}
	else
		{
		QFile file( lastFilename ); // Write the text to a file
   		 if ( file.open( IO_WriteOnly ) ) 
			{
        		QTextStream stream( &file );
        		stream << textEdit1->text();
        		textEdit1->setModified( FALSE );
			setCaption( tr( "PNC3Comm - " + lastFilename ) );
   			}
		}

}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::fileSaveAs()
{
    QString filename = QFileDialog::getSaveFileName(
                    defaultDir,
                    "CNC Files (*.cnc *.CNC *.nc *.NC *.ngc *.NGC);;"
                    "All Files (*.*)",
                    this,
                    "Save File",
                    "Choose a filename to save under" );
	if(!filename.isEmpty())
		{
 		QFile file( filename ); // Write the text to a file
    		if ( file.open( IO_WriteOnly ) ) 
                {
        		QTextStream stream( &file );
        		stream << textEdit1->text();
        		textEdit1->setModified( FALSE );
			    setCaption( tr( "PNC3Comm - " + filename ) );
                lastFilename = filename;
    			}
		}
    
}

/////////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::fileExit()
{
	if(textEdit1->isModified())
		{
		fileSave();
		}
	qApp->quit();

//debug to test	FormatReturn();
}

/****************************************************************************************************/


void PNC3::editUndo()
{
    textEdit1->undo();
}
void PNC3::editRedo()
{
    textEdit1->insert("Error");
    textEdit1->redo();
}

void PNC3::editCut()
{
    textEdit1->cut();
}

void PNC3::editCopy()
{
    textEdit1->copy();
}

void PNC3::editPaste()
{
    textEdit1->paste();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::OnClear()
{
int x;
	fileNew();
	textEdit2->setText("");
	blockList.clear();
	for(x=0; x < 100; x++)
		cmdstrings[x].clear();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::gcodePCD()
{
    Form1 *w = new Form1;
    w->exec();
}


void PNC3::gcodeHoles()
{
    Form2 *w = new Form2;
    w->exec();
}

void PNC3::gcodeRings()
{
    Form4 *w = new Form4;
    w->exec();

}

void PNC3::gcodeRectangle()
{
    Form5 *w = new Form5;
    w->exec();

}


void PNC3::peckSettings()
{
 
   Form3 *w = new Form3;

//    std::cout  << "PNC3 Before " << peckRatio << feedRatio << speedRatio << minPeckRatio << maxPeckRatio << minSpeedRatio << minFeedRatio << "\n" << endl;

    w->exec();

//    std::cout  << "PNC3 After " << peckRatio << feedRatio << speedRatio << minPeckRatio << maxPeckRatio << minSpeedRatio << minFeedRatio << "\n" << endl;
}

void PNC3::helpAbout()
{
    QMessageBox::information(this, "PNC3Comm - About",
            "Published under GPL licence.\n\n"
            "Version 0.2-beta\n\n"
            "(c) MGWare 2010." );
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::OnSend_clicked()
{
QString s1, s2, s3;
QString buff;
QString filename;
int x, z, linenum;
char buf[8];

	x = z = 0;
    linenum = 1;

    fileSave();   // if it has been altered since loading will force a save
   
    if(!lastFilename.isEmpty())
		{
        textEdit1->setText("");

		QFile file( lastFilename ); // Read the text from a file
		file.open(IO_ReadOnly);
		QTextStream stream(&file);

		blockList.clear();

		while(!stream.atEnd())
			{
			buff = stream.readLine();
			Format(buff, true);
			if(buff.length())
				{// insert line numbers unles at M2
                if((buff.find("N", 0) == -1) && ( (buff.find("M2", 0) == -1) && (buff.find("M02", 0) == -1) ) )
                    {
                    s1 = QString("N%1 ")
                        .arg(linenum++, 0, 'f', 0);
                    buff = s1 + buff;
                    }
				blockList += buff;
				textEdit1->insert(buff);
				}
			}
		setCaption( tr( "PNC3Comm - " + lastFilename ) );
//        textEdit2->insert("Data sent"); //DEBUG
		}   	
    else
        {       
        textEdit2->setText("Filename Error");
        return;
        }

	fd = portSetUp();
        if(fd==-1) 
		{
		textEdit2->setText("Error opening port");
		return;
		}

	textEdit2->setText("");

    	for(x=0; x < 100; x++)
		cmdstrings[x].clear();
	
	// The gcode string in *it had a newline \n on the end of it and this caused all the problems
	// with the PNC3 not recognising the string and sending <NAK>

	for ( QStringList::Iterator it = blockList.begin(); it != blockList.end(); ++it)
		{	
		s1 = *it;
		s1 = s1.stripWhiteSpace(); // get rid of \n
		s2 = "\x02 ";
		s2 += s1;
		s2 += "\r\n\x03";
		cmdstrings[z].append(s2);
		z++;
		}		
	// for each QStrList entry
	for(x = 0; x < z; x++)
		{
		s2 = cmdstrings[x].at(0);
		portWrite(s2);
		usleep(500 * 1000);  // fastest that PNC3 able to reliably receive all lines

		if (read (fd, buf, 4) != -1)
			{
		   	buf[4]='\0';
			if(strlen(buf))  // don't check if returns zeros
				{
				if(checkReturn(buf))  // error occurred
					{
    				textEdit2->insert(s2 = "Sent - " + s2 + "\n");    		
					textEdit2->insert(s2.sprintf("Error on line %d",x+1));
					return;
					}
				}
			}
		}
	textEdit2->insert("\nFinished transmission\n");
	
}

bool PNC3::checkReturn(char * buf)
{
QString s1, s2, cstr;
int retval;
bool bError, bRet;

	cstr = "";
	bError = false;
//  For some reason read is returning 6 instead of 06 for ACK
//  Probably because the 0 would be seen as \0 string termination
//  This caused lots of problems until this conversion and test was arrived at	
    s1.sprintf("%x",buf[0]);

    if(s1 == "6")
	    cstr= "<ACK>\n";
	if(s1 == "15")
	    {    
	    cstr= "<NAK>\n";    
	    bError = true;
	    }
	if(s1 == "ec" || s1 == "EC")
	    cstr= "<Not ready>\n";
	if(s1 == "3f" || s1 == "3F")
	    cstr= "<?>\n";    
	
	if(! cstr.length())
	    cstr= "Return not recognised\n";

	if(bError)
		{
		s1 = buf;
		s1 = s1.right(s1.length() -1);
		s1 = s1.left(2);
		retval = s1.toInt(&bRet,16);

		getErrorCode(s2, retval);
		cstr += s2 + "\n";
		}	

	cstr = "Received:- " + cstr ;
		
	textEdit2->insert(cstr);

	return bError;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////



void PNC3::OnReceive_clicked()
{
QString s2;
char buf[4];

    s2 = "";

	fd = portSetUp();
        if(fd==-1) 
		{
		textEdit2->setText("Error opening port");
		return;
		}

	OnClear();  // will be using main edit window to receive data

	
	// initial ack to get PNC3 starting transmit
   	portWrite(ack);   
   	usleep(100 * 1000);  

	while(s2.find("M02") == -1)
		{
		portRead(buf);
        	s2.append(buf);
		}	
	// send a final <ACK>s so that PNC3 exits transmit loop after M02
	portWrite(ack);
	usleep(100 * 1000); 
	portWrite(ack);
    
    textEdit1->setText(s2);

	FormatReturn();
	
//	if(fd != -1)
//		close(fd);
}




///////////////////////////////////////////////////////////////////////////////
// the simplest and by far quickest read method is 1 byte at a time
// <ACK> is sent from within read function

int PNC3::portRead(char* buf)
{
int n = 0;
int p = 0;

    
    while(n < 1)
        {
        p = read(fd, buf, 1);
        if(p != -1)
            n += p; 
        portWrite(ack);
        }
    buf[1] = 0; 
    
    return n;
}



int PNC3::portWrite(const char* str)
{
    int len = strlen(str);
    int n = write(fd, str, len);
    if( n!=len ) 
        return -1;
    return 0;
}


///////////////////////////////////////////////////////////////////////////////////



void PNC3::getErrorCode(QString& s3, int retval)
{

	switch(retval)
		{
		case 1:
			s3 = "Error 01 Not Assigned";
			break;
		case 2:
			s3 = "Error 02 Illegal G code received";
			break;
		case 3:
			s3 = "Error 03 Illegal M code received";
			break;
		case 4:
			s3 = "Error 04 Illegal character for this block";
			break;
		case 5:
			s3 = "Error 05 Move exceeds machine limits";
			break;
		case 6:
			s3 = "Error 06 Block not completed successfully";
			break;
		case 7:
			s3 = "Error 07 This block not allowed to execute immediately (program only block)";
			break;
		case 8:
			s3 = "Error 08 This block not allowed in a program";
			break;
		case 9:
			s3 = "Error 09 Attempt to run non existant block";
			break;
		case 10:
			s3 = "Error 10 PNC memory full";
			break;
		case 11:
			s3 = "Error 11 Block too big for input buffer";
			break;
		case 12:
			s3 = "Error 12 X axis drive system fault";
			break;
		case 13:
			s3 = "Error 13 Y axis drive system fault";
			break;
		case 14:
			s3 = "Error 14 Z axis drive system fault";
			break;
		case 15:
			s3 = "Error 15 W axis drive system fault";
			break;
		case 16:
			s3 = "Error 16 Incomplete block received";
			break;
		case 17:
			s3 = "Error 17 Error in input co-ordinate";
			break;
		case 18:
			s3 = "Error 18 error in input feedrate";
			break;
		case 19:
			s3 = "Error 19 X and Y moves not present in circular move";
			break;
		case 20:
			s3 = "Error 20 Position not known, machine must be driven to datum";
			break;
		case 21:
			s3 = "Error 21 Circular move not within a quadrant";
			break;
		case 22:
			s3 = "Error 22 Dwell value error";
			break;
		case 23:
			s3 = "Error 23 Scale value error";
			break;
		case 24:
			s3 = "Error 24 Tool number error";
			break;
		case 25:
			s3 = "Error 25 Auxiliary selection error";
			break;
		case 26:
			s3 = "Error 26 Input selection error";
			break;
		case 27:
			s3 = "Error 27 Repeat start block error";
			break;
		case 28:
			s3 = "Error 28 Repeat end block error";
			break;
		case 29:
			s3 = "Error 29 Number of repeats error";
			break;
		case 30:
			s3 = "Error 30 Nest error in repeat levels";
			break;
		case 31:
			s3 = "Error 31 Error in repeat offsets";
			break;
		case 32:
			s3 = "Error 32 Error in offset block";
			break;
		case 33:
			s3 = "Error 33 Splidle speed input exceeds limits";
			break;
		case 34:
			s3 = "Error 34 Spindle direction is opposite to present direction, STOP spindle first";
			break;
		case 35:
			s3 = "Error 35 Error when driving to datum";
			break;
		case 36:
			s3 = "Error 36 Threading input error";
			break;
		case 37:
			s3 = "Error 37 Spindle speed wrong for threading";
			break;
		case 38:
			s3 = "Error 38 Spindle drive system error";
			break;
		case 39:
			s3 = "Error 39 Number of cycles error";
			break;
		default:
			s3 = "Invalid error code";
			break;
		}


}


void PNC3::FormatReturn()
{
QString buff;

QString filename = "/tmp/return.tmp";

	
	// quick and dirty way to get a return which is divided into blocks based on CR/LF chars
	// write the buffer to a file and then read it back line by line

	QFile file( filename ); // Write the text to a file
   	if ( file.open( IO_WriteOnly ) ) 
		{
        	QTextStream stream( &file );
        	stream << textEdit1->text();
        	textEdit1->setText("");
		}
	file.close();
	
	QFile file2( filename ); // Write the text to a file
   	if ( file2.open( IO_ReadWrite ) ) 
		{
		QTextStream stream( &file2 );
        
		while(!stream.atEnd())
			{
			buff = stream.readLine();
			Format(buff, false);
			if(buff.length())
				textEdit1->insert(buff);
			}
		}
	file2.close();


}
///////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::Format(QString& buff, bool bOpen)
{
QString s1, s2;
int x = 0;

	if(bOpen)
		{
		buff = buff.stripWhiteSpace();
		buff = buff.simplifyWhiteSpace();

		if((x = buff.find(";")) != -1)
			{
			if(x == 0)
				buff = "";
			else
				buff = buff.left(x-1);
			}

		if((x = buff.find("(")) != -1)
			{
			if(x == 0)
				buff = "";
			else
				buff = buff.left(x-1);
			}
	
		if(buff.length())
			{
			if((buff.at(buff.length() -1)) != QChar('L'))
				buff += "L";
			buff += "\n";
			}
		}
	else
		{
		buff = buff.stripWhiteSpace();
		buff = buff.simplifyWhiteSpace();

		// get the start of the line sorted first
		if((x = buff.find("N")) != -1)
			{
			while((x > 0) && (x != -1))
				{
				buff = buff.right(buff.length() - 1);	
				x = buff.find("N");
				}
			buff += "\n";
			}
		else
			{
			if((x = buff.find("M02")) != -1)  // PNC3 uses M02 to signify end of program but does not use line no
				{
				while((x > 0) && (x != -1))
					{
					buff = buff.right(buff.length() - 1);	
					x = buff.find("M02");
					}
				}
			else
				buff = "";
			}
		
	
		if(buff.length())
			{
			removeChar(buff, QString("+"));  // all values are implicitly pos unless signed neg
							 // would not be a problem except that none 0x2b chars also
							 // get represented as + in ascii, so remove all		
			spaceChar(buff, QString("G"));
			spaceChar(buff, QString("M"));
			spaceChar(buff, QString("X"));
			spaceChar(buff, QString("Y"));
			spaceChar(buff, QString("Z"));
			spaceChar(buff, QString("R"));
			spaceChar(buff, QString("I"));
			spaceChar(buff, QString("J"));
			spaceChar(buff, QString("K"));
			spaceChar(buff, QString("E"));
			spaceChar(buff, QString("R"));
			spaceChar(buff, QString("N"));
			spaceChar(buff, QString("S"));
			spaceChar(buff, QString("F"));						

			buff = buff.stripWhiteSpace();  // after spacing N used in repeat loops will also move line number so get rid of space
			buff += "\n";

			removeNextChar(buff, QString("F"));  // close gap between F Z and F X
			}
		}	

}

///////////////////////////////////////////////////////////////////////////////////////////////////

void PNC3::spaceChar(QString& str, QString c)
{
int x = 0;

	while((x = str.find(c, x)) != -1)
		{
		str = str.left(x) + " " + (str.right(str.length() - (x )));
		x += 3;
		}
    
}


void PNC3::removeChar(QString& str, QString c)
{
int x;
QString s1, s2;

	x = 0;

	while((x = str.find(c, x)) != -1)
		{
		s1 = str.left(x);
		s2 = str.right(str.length() - (x+1));
		str = s1 + s2;
		x--;		
		}
}

void PNC3::removeNextChar(QString& str, QString c)
{
int x;
QString s1, s2;

	x = 0;

	while((x = str.find(c, x)) != -1)
		{
		s1 = str.left(++x);
		s2 = str.right(str.length() - (x+1));
		str = s1 + s2;
		}
}

//////////////////////////////////////////////////////////////////////////////////////////////

int PNC3::portSetUp()
{
    struct termios toptions;
    int fd;
    
    fd = open("/dev/ttyS0", O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1) 
        return -1;
    
    
    if (tcgetattr(fd, &toptions) < 0) 
        return -1;
    
    speed_t brate = B4800; 
   
    cfsetispeed(&toptions, brate);
    cfsetospeed(&toptions, brate);

    // 7E2
    toptions.c_cflag |= PARENB;  // even
    toptions.c_cflag |= CSTOPB;  //2
    toptions.c_cflag &= ~CSIZE;  // bit mask
    toptions.c_cflag |= CS7;     // 7
    // no flow control
    toptions.c_cflag &= ~CRTSCTS;

    toptions.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    toptions.c_iflag = ICRNL | IGNPAR;// turn off s/w flow ctrl
 

    toptions.c_oflag = 0;
    toptions.c_lflag = 0;
    
    if( tcsetattr(fd, TCSANOW, &toptions) < 0) 
        return -1;
    
    return fd;
}
