#include "Ring_dlg.h"

#include <math.h>
#include <qmessagebox.h>

void Form4::On_Quit_clicked()
{
    close();
}

void Form4::On_generate_clicked()
{
 QString cstr;
double   X_centre, Y_centre, inter_pos_feed, cutting_feed;
double   Z_boring_feed, Z_inter_pos_height, Z_bore_depth, Z_start_height, Z_decrement;
double   cutter_diameter,  spindle_speed, ring_diameter, cut_radius, subcut_radius, cuts;

int     linenum = 1;
int     block_start, block_end, f;
bool    b_coolant, b_pocket;



    // get data from line edits

    X_centre = ((QString)le_X_centre->text()).toDouble();
    Y_centre = ((QString)le_Y_centre->text()).toDouble();
    inter_pos_feed = ((QString)le_inter_position_feed->text()).toDouble();
    Z_boring_feed =  ((QString)le_Z_boring_feed->text()).toDouble();  
    Z_inter_pos_height = ((QString)le_Z_inter_position_height->text()).toDouble();
    Z_bore_depth = ((QString)le_Z_bore_depth->text()).toDouble();
    if(Z_bore_depth < 0)
        Z_bore_depth = Z_bore_depth * -1;   // make sure not neg number
    Z_start_height = ((QString)le_Z_start_height->text()).toDouble();
    Z_decrement = ((QString)le_Z_decrement->text()).toDouble();
    cutter_diameter = ((QString)le_cutter_diameter->text()).toDouble();
    cutting_feed  = ((QString)le_cutting_feed->text()).toDouble();
    spindle_speed = ((QString)le_spindle_speed->text()).toDouble();
    ring_diameter = ((QString)le_ring_diameter->text()).toDouble();

    b_pocket = rb_pocket->isChecked();

    block_start = block_end = 0; 
    f = 0;
    b_coolant = cb_coolant->isChecked();

    cut_radius = ( ring_diameter / 2) - (cutter_diameter /2 );

    cuts = floor(Z_bore_depth / Z_decrement);

    if((cuts * Z_decrement) != Z_bore_depth)
        {
        QMessageBox::information(this, "EASIMILL - Ring Milling generator",
            "Z decrement does not divide equally into required Z final depth\n"
            "Will be rounded down so may not break through or reach required depth\n"
            "For exact depth, substitute for a figure that will divide equally");
        }
     if(X_centre < 0 || Y_centre < 0)
        {
        QMessageBox::information(this, "EASIMILL - Rectangle Milling generator",
            "X and Y centre co-ordinates can be Zero or a positive offset\n"
            "If you want to start at a negative offset, change your G55 offsets accordingly");
        return;
        }
    ///////   start strings

    cstr.sprintf("N%d M3 S%.0f \n",linenum++, spindle_speed);
    te_gcode->setText(cstr);
    cstr.sprintf("N%d G01 X%.3f Y%.3f FX%.0f Z%.3f FZ%.0f\n",linenum++, X_centre, Y_centre, inter_pos_feed, Z_start_height, inter_pos_feed);
    te_gcode->append(cstr);

    cstr.sprintf("N%d G01 X%.3f Y%.3f FX%.0f Z%.3f FZ%.0f\n",linenum++ ,X_centre, cut_radius + Y_centre, inter_pos_feed, Z_inter_pos_height, inter_pos_feed);
    te_gcode->append(cstr);
    
    if(b_coolant)
        {
        cstr.sprintf("N%d M8\n", linenum++);
        te_gcode->append(cstr);
        }    
        
    block_start = linenum;

    // First line of block must contain co-ordinates for all axes as machine returns to this position
    // at every iteration +/- offsets in G81 block

    cstr.sprintf("N%d G01 X%.3f Y%.3f FX%.0f Z-%.3f FZ%.0f\n",linenum++ , X_centre, cut_radius + Y_centre, cutting_feed, Z_decrement , Z_boring_feed);
    te_gcode->append(cstr);
    
    cstr.sprintf("N%d G02 X%.3f Y%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , cut_radius + X_centre, Y_centre, cutting_feed, X_centre, Y_centre);  //G02 X110.000 Y0.000C X0.000 Y0.000L
    te_gcode->append(cstr);

    cstr.sprintf("N%d G02 X%.3f Y-%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , X_centre, cut_radius - Y_centre, cutting_feed, X_centre, Y_centre);  // G02 X0.000 Y-110.000C X0.000 Y0.000L
    te_gcode->append(cstr);

    cstr.sprintf("N%d G02 X-%.3f Y%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , cut_radius - X_centre, Y_centre,cutting_feed, X_centre, Y_centre);  //G02 X-110.000 Y0.000C X0.000 Y0.000L
    te_gcode->append(cstr);

    cstr.sprintf("N%d G02 X%.3f Y%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , X_centre, cut_radius + Y_centre, cutting_feed, X_centre, Y_centre);  // G02 X0.000 Y110.000C X0.000 Y0.000L
    te_gcode->append(cstr);

    if(b_pocket)
        {
        subcut_radius = cut_radius - (cutter_diameter - 1);
        while(subcut_radius > (cutter_diameter /2 ))
            {
            cstr.sprintf("N%d G01 X%.3f Y%.3f FX%.0f\n",linenum++ , X_centre, subcut_radius + Y_centre,cutting_feed);
            te_gcode->append(cstr);
    
            cstr.sprintf("N%d G02 X%.3f Y%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , subcut_radius + X_centre, Y_centre, cutting_feed,X_centre, Y_centre);  //G02 X110.000 Y0.000C X0.000 Y0.000L
            te_gcode->append(cstr);

            cstr.sprintf("N%d G02 X%.3f Y-%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , X_centre, subcut_radius - Y_centre, cutting_feed, X_centre, Y_centre);  // G02 X0.000 Y-110.000C X0.000 Y0.000L
            te_gcode->append(cstr);

            cstr.sprintf("N%d G02 X-%.3f Y%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , subcut_radius - X_centre, Y_centre, cutting_feed, X_centre, Y_centre);  //G02 X-110.000 Y0.000C X0.000 Y0.000L
            te_gcode->append(cstr);

            cstr.sprintf("N%d G02 X%.3f Y%.3f FX%.0fC X%.3f Y%.3f\n",linenum++ , X_centre, subcut_radius + Y_centre, cutting_feed, X_centre, Y_centre);  // G02 X0.000 Y110.000C X0.000 Y0.000L
            te_gcode->append(cstr);
            
            subcut_radius = subcut_radius - (cutter_diameter - 1);
            }
        // once the piece left is smaller than the cutter, move cutter to circle centre to remove it
        cstr.sprintf("N%d G01 X%.3f Y%.3f FX%.0f\n",linenum++ , X_centre, Y_centre, cutting_feed);
        te_gcode->append(cstr);
        }

    block_end = linenum - 1;

    if((cuts - 1) > 0)
        {
        cstr.sprintf("N%d G81 R%d E%d N%.0f X0 Y0 Z-%.3f FZ%.0f\n",linenum++, block_start, block_end, cuts -1, Z_decrement, Z_boring_feed);
        te_gcode->append(cstr);
        }

////////////    end strings

    cstr.sprintf("N%d G00 Z%.3f\n" , linenum++, Z_start_height);
    te_gcode->append(cstr);
    if(b_coolant)
        {
        cstr.sprintf("N%d M9\n", linenum++);
        te_gcode->append(cstr);
        }    
    cstr.sprintf("N%d M5\nM2\n", linenum++ ); 
    te_gcode->append(cstr);

    te_gcode->selectAll(true);
    te_gcode->copy();

    cstr.sprintf("\n\nThis code is now in the clipboard\n\n");
    te_gcode->append(cstr);   
}



